/****************************************************************************
** Meta object code from reading C++ file 'settingsmodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../krecorder-26.07.80/src/settingsmodel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'settingsmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13SettingsModelE_t {};
} // unnamed namespace

template <> constexpr inline auto SettingsModel::qt_create_metaobjectdata<qt_meta_tag_ZN13SettingsModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SettingsModel",
        "simpleAudioFormatChanged",
        "",
        "audioCodecChanged",
        "containerFormatChanged",
        "audioQualityChanged",
        "simpleAudioFormat",
        "audioCodec",
        "containerFormat",
        "audioQuality",
        "SimpleAudioFormat",
        "VORBIS",
        "OPUS",
        "FLAC",
        "MP3",
        "WAV",
        "OTHER"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'simpleAudioFormatChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'audioCodecChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'containerFormatChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'audioQualityChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'simpleAudioFormat'
        QtMocHelpers::PropertyData<int>(6, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'audioCodec'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'containerFormat'
        QtMocHelpers::PropertyData<QString>(8, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'audioQuality'
        QtMocHelpers::PropertyData<int>(9, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'SimpleAudioFormat'
        QtMocHelpers::EnumData<enum SimpleAudioFormat>(10, 10, QMC::EnumFlags{}).add({
            {   11, SimpleAudioFormat::VORBIS },
            {   12, SimpleAudioFormat::OPUS },
            {   13, SimpleAudioFormat::FLAC },
            {   14, SimpleAudioFormat::MP3 },
            {   15, SimpleAudioFormat::WAV },
            {   16, SimpleAudioFormat::OTHER },
        }),
    };
    return QtMocHelpers::metaObjectData<SettingsModel, qt_meta_tag_ZN13SettingsModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SettingsModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13SettingsModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13SettingsModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13SettingsModelE_t>.metaTypes,
    nullptr
} };

void SettingsModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SettingsModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->simpleAudioFormatChanged(); break;
        case 1: _t->audioCodecChanged(); break;
        case 2: _t->containerFormatChanged(); break;
        case 3: _t->audioQualityChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SettingsModel::*)()>(_a, &SettingsModel::simpleAudioFormatChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsModel::*)()>(_a, &SettingsModel::audioCodecChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsModel::*)()>(_a, &SettingsModel::containerFormatChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsModel::*)()>(_a, &SettingsModel::audioQualityChanged, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->simpleAudioFormat(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->audioCodec(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->containerFormat(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->audioQuality(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSimpleAudioFormat(*reinterpret_cast<int*>(_v)); break;
        case 1: _t->setAudioCodec(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setContainerFormat(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setAudioQuality(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *SettingsModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SettingsModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13SettingsModelE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int SettingsModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void SettingsModel::simpleAudioFormatChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void SettingsModel::audioCodecChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void SettingsModel::containerFormatChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void SettingsModel::audioQualityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
