/*
 * SPDX-FileCopyrightText: 2020 Jonah Brüchert <jbb@kaidan.im>
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 */

#pragma once

#include <QObject>

class Utils : public QObject
{
    Q_OBJECT

public:
    explicit Utils(QObject *parent = nullptr);

    Q_INVOKABLE static QString formatDateTime(const QString &time);
    Q_INVOKABLE static QString formatTime(int time);
    Q_INVOKABLE static QString formatDuration(int duration);
};
